import { Button, Col, Popconfirm, Result, Row } from "antd";
import { useEffect, useRef, useState } from "react";
import GroupTasksViewModal from "./GroupTasksViewModal";
import GroupTypeSelectionModal from "./GroupTypeSelectionModal";
import GroupTaskTableList from "./GroupTasksTableList";
import {
  Constants,
  SentMessagesCommands,
  hasPermission,
  myFetch,
} from "../../../utils";
import { useTranslation } from "react-i18next";
import { ReloadOutlined } from "@ant-design/icons";
import { useGroupTasksContext } from "../../../Contexts/GroupTasksContext";
import { useAppContext } from "../../../Contexts/AppContext";
import { useWebSocketContext } from "../../../Contexts/WebSocketContext";
import { useParams } from "react-router-dom";
import MyPagination from "../../../Components/MyPagination";

export default function GroupTasks({ isGroupTasksViewModalOpen }) {
  const webSocketContext = useWebSocketContext();
  const appContext = useAppContext();
  const groupTasksContext = useGroupTasksContext();
  const { paramCategory } = useParams();

  const [isGroupTypeSelectionModalOpen, setIsGroupTypeSelectionModalOpen] =
    useState(false);
  const [currentCategoryGroup, setCurrentCategoryGroup] = useState([]);
  const [currentSelectedModalGroupType, setCurrentSelectedModalGroupType] =
    useState();
  const { t } = useTranslation();
  const [paginationPage, setPaginationPage] = useState(1);
  const totalPages = useRef(0);

  const showGroupTypeSelectionModal = (categoryGroup) => {
    setCurrentCategoryGroup(categoryGroup);
    setIsGroupTypeSelectionModalOpen(true);
    setCurrentSelectedModalGroupType(null);
  };

  const onCheckingForCategoryGroupChanges = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.GroupTasksCheckingForCategoryGroupChanges,
      {}
    );
  };

  const onInstallGlobalPythonPackages = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.GroupTasksInstallGlobalPythonPackages,
      {}
    );
  };

  const fetchGroupTasks = (page) => {
    myFetch(`/grouptasks/${paramCategory}?page=${page}`, "GET").then((data) => {
      totalPages.current = data.TotalPages;

      groupTasksContext.setCategoryGroup(data.CategoryGroup);
      groupTasksContext.setGroupTasks(data.GroupTasks);
    });
  };

  useEffect(() => {
    if (paginationPage === 1) {
      fetchGroupTasks(1);
    } else {
      setPaginationPage(1);
    }
  }, [paramCategory]);

  useEffect(() => fetchGroupTasks(paginationPage), [paginationPage]);

  return (
    <>
      {groupTasksContext.categoryGroup.Category === "" ||
      groupTasksContext.categoryGroup.groups === null ? (
        <Result status="404" title={t("groupTasks.categoryGroups.length0")} />
      ) : (
        <>
          <Row>
            <Col
              xs={24}
              sm={{ span: 10, offset: 14 }}
              md={{ span: 8, offset: 16 }}
            >
              {hasPermission(
                appContext.userPermissions,
                Constants.PERMISSIONS.GROUP_TASKS.INSTALL_GLOBAL_PYTHON_PACKAGES
              ) && (
                <div>
                  <Popconfirm
                    placement="top"
                    okText={t("common.button.confirm")}
                    cancelText={t("common.button.cancel")}
                    title={t(
                      "groupTasks.button.installGlobalPythonPackages.popover.title"
                    )}
                    onConfirm={() => onInstallGlobalPythonPackages()}
                  >
                    <Button icon={<ReloadOutlined />} block>
                      {t("groupTasks.button.installGlobalPythonPackages.title")}
                    </Button>
                  </Popconfirm>
                </div>
              )}
            </Col>
          </Row>

          {Object.keys(groupTasksContext.categoryGroup).length > 0 && (
            <>
              <GroupTaskTableList
                categoryGroup={groupTasksContext.categoryGroup}
                showGroupTypeSelectionModal={showGroupTypeSelectionModal}
                groupTasks={groupTasksContext.groupTasks}
              />

              <MyPagination
                paginationPage={paginationPage}
                setPaginationPage={(page) => setPaginationPage(page)}
                totalPages={totalPages.current}
              />

              {hasPermission(
                appContext.userPermissions,
                Constants.PERMISSIONS.GROUP_TASKS
                  .CHECKING_FOR_CATEGORY_GROUP_CHANGES
              ) && (
                <div
                  style={{
                    display: "flex",
                    justifyContent: "center",
                    marginTop: 20,
                  }}
                >
                  <Popconfirm
                    placement="top"
                    okText={t("common.button.confirm")}
                    cancelText={t("common.button.cancel")}
                    title={t(
                      "groupTasks.button.checkingForCategoryGroupChanges.popover.title"
                    )}
                    onConfirm={() => onCheckingForCategoryGroupChanges()}
                  >
                    <Button
                      shape="round"
                      icon={<ReloadOutlined />}
                      size="large"
                    >
                      {t(
                        "groupTasks.button.checkingForCategoryGroupChanges.check"
                      )}
                    </Button>
                  </Popconfirm>
                </div>
              )}
            </>
          )}
        </>
      )}

      <GroupTypeSelectionModal
        isOpen={isGroupTypeSelectionModalOpen}
        setIsOpen={setIsGroupTypeSelectionModalOpen}
        categoryGroup={currentCategoryGroup}
        currentSelectedModalGroupType={currentSelectedModalGroupType}
        setCurrentSelectedModalGroupType={setCurrentSelectedModalGroupType}
      />

      <GroupTasksViewModal isOpen={isGroupTasksViewModalOpen} />
    </>
  );
}
